#
# init.tcl - set variables, parse arguments, and initialize the . window
#
# $Header: /nfs/home/src/base/local/tk/pgb3.0/lib/RCS/init.tcl,v 1.2 1993/12/30 01:12:52 jimbo Exp jimbo $
#


# set the Home of PGBROWSE
if [info exists env(PGBROWSE)] {
	set PGB(Home) $env(PGBROWSE)
} else {
	set PGB(Home) /usr/local/pgbrowse
}

# check for and append the  PGBrowse library
if ![file exists $PGB(Home)/lib/browse.tcl] {
	puts stderr "
Error:
	$PGB(Home) is not the PGBROWSE installation directory.
	Please set PGBROWSE to the correct location.
"
	exit 1
}
lappend auto_path $PGB(Home)/lib

# the pgbrowse bin/ directory normally contains subdirectories
# for the various architecture types
if [info exists env(ARCH)] {
    set PGB(Bin) $PGB(Home)/bin/$env(ARCH)
} else {
    set PGB(Bin) $PGB(Home)/bin
}

# the initial database host
if [info exists env(PGHOST)] {
    set host $env(PGHOST)
} else {
    set host heel.s2k.berkeley.edu
}

# the initial port number
if [info exists env(PGPORT)] {
    set port $env(PGPORT)
} else {
    set port 4321
}

# the initial database
if [info exists env(PGDATABASE)] {
	set db $env(PGDATABASE)
} elseif [info exists env(DATABASE)] {
    set db $env(DATABASE)
} else {
    set db ""
}

set class {}
set query {}

if [info exists env(PGBACTIONINIT)] {
    set PGB(actionInit) $env(PGBACTIONINIT)
} else {
    set PGB(actionInit) pgb.tcl
}

# the options file normally just contains which fields to retrieve
# for a class in the query tool
if [info exists env(PGBOPTIONS)] {
    set PGB(optionFile) $env(PGBOPTIONS)
} else {
    set PGB(optionFile) ~/.pgbrowse
}

# read the options file
# if [file exists $PGB(optionFile)] {source $PGB(optionFile)}

# parse comand line args
if [catch {Tku_Args {debug log hide verify nomodify system big :host :db :class :query :port} $argv}] {
	puts stderr "Usage:
	pgbrowse -host host -port port -db database -class class -query -nomodify
Where:
  -host host	Postgres Server
			(default: \$PGHOST or heel.s2k.berkeley.edu)
  -port port	TCP port number (default: \$PGPORT or 4321)

  -db database	Database to open

  -class class\[,class,...\]
		List of class(es) to open

  -query class\[,class,...\]
		List of class(es) to query

  -nomodify	Do not allow adding to or modifying attribute options
  -hide		Hides the toplevel pgbrowse window
  -system	Include system classes in the classes listbox
  -verify	Verify all Postquel commands
"
	exit 1
}

# save boolean arguements in PGB
set PGB(-nomodify) $nomodify
set PGB(-debug) $debug
set PGB(-system) $system
set PGB(-verify) $verify
set PGB(-hide) $hide

if $big {
	option add *Font -adobe-helvetica-bold-r-normal-*-*-180-*
	option add *Entry.font -adobe-courier-medium-r-normal-*-*-180-*
	option add *Text.font -adobe-courier-medium-r-normal-*-*-180-*
	option add *fixedfont -adobe-courier-bold-r-normal-*-*-180-*
	option add *Pad 10
} else {
	option add *fixedfont -adobe-courier-bold-r-normal-*-*-120-*
}

# don't allow the browser to be hidden when we don't have enough info
if {$PGB(-hide) && $db == ""} {set PGB(-hide) 0}
if {$PGB(-hide) && $class == "" && $query == ""} {set PGB(-hide) 0}

# Initialize the log window
    toplevel .log
    wm withdraw .log
    wm title .log "Postquel Command Log"
    wm minsize .log 100 100 
    wm geom .log +1-1
    frame .log.frame
    text .log.text -yscroll {.log.vs set} -state disabled -wrap word
    scrollbar .log.vs -command {.log.text yview}
    Tku_MenuBar .log.menu \
    	{About {This window displays the commands sent to the
    	Postgres backend and the messages returned.  You can
    	save the log at anytime and can add notes by clicking
    	at the desired location and typing}}
    
    Tku_MenuButton .log.menu "File -under 0" \
    	{command -label "Save Log" -command LogSave -accel Ctrl-S -under 0} \
    	    {Save the log to a file} \
    	{command -label "Clear Log" -command LogClear -accel Ctrl-X -under 1} \
    	    {Clears the log window} \
    	{command -label Close -command LogClose -accel Ctrl-W -under 0} \
    	    {Closes the log window}
    
    Tku_StatusBar .log.info "PGBrowse Log"
    .log.info conf 
    pack .log.menu .log.info -fill x
    pack .log.text -side left -fill both -expand 1 -in .log.frame
    pack .log.vs -side left -fill y -in .log.frame
    pack .log.frame -fill both -expand 1
    bind .log.text <Button-1> {focus %W}
    bind .log.text <FocusIn> {%W conf -relief sunken}
    bind .log.text <FocusOut> {%W conf -relief raised}
    bind .log.text <Map> {focus .log.text}
    bind .log.text <Unmap> {focus [focus default]}
    bind .log <FocusIn> {focus .log.text}
    bind .log <FocusOut> {focus [focus default]}
    wm protocol .log WM_DELETE_WINDOW LogClose


# Hide or initialized the root window

   # set window manager stuff
    wm command . [concat pgbrowse $argv]
    wm client . [exec hostname]
    wm title . "PGBrowse"
    wm protocol . WM_DELETE_WINDOW CloseBrowser
    wm geom . +75+75

if $PGB(-hide) {

    if $log {
	wm withdraw .
    } else {
    	Tku_StatusBar .info PGBrowse
    	.info.status conf -width 40
    	pack .info -fill x
    }

} else {

    wm minsize . 30 10
    wm maxsize . 30 50

    Tku_StatusBar .info PGBrowse

    Tku_MenuBar .menu {About {PGBrowse Version 3.0Beta}}
    
    Tku_MenuButton .menu "File -under 0" \
    	"command -label {Open} -command File:Open -accel Ctrl-O -under 0" \
    	    {Opens the selected database or class(s)} \
    	"command -label {New Browser} -command File:NewBrowser -accel Ctrl-B -under 0" \
    	    {Opens a new PGBROWSE window} \
    	"command -label {Set PGHOST} -command SetPGHost -accel Ctrl-H -under 6" \
    	    {Changes the Postgres server hostname} \
    	"command -label {Set PGPORT} -command SetPGPort -accel Ctrl-P -under 6" \
    	    {Changes the Postgres IP port} \
    	"command -label {Show Log} -command LogOpen -accel Ctrl-L -under 5" \
    	    {Shows the log of monitor commands} \
    	separator {} \
    	"command -label Close -command CloseBrowser -accel Ctrl-W -under 0" \
    	    "Close this browser"

    Tku_MenuButton .menu "Edit -under 0" \
    	"command -label {New Database} -command CreateDB -under 0" \
    	    {Creates a new database on the current server} \
    	"command -label {New Class} -command CreateClass -under 4" \
    	    {Create a new class in the current database} \
    	"command -label {Add Attribute} -command ModClass -under 4" \
    	    {Add attributes to a class} \
    	"command -label {Empty Class} -command ClearClass -under 0" \
    	    {Deletes all instances from a class} \
    	"command -label {Destroy Class} -command DestroyClass -under 0" \
    	    {Destroys a class} \
    	"command -label {Send Postquel} -command InteractivePQ -under 5" \
    	    {Sends a Postquel command to the backend}
    
    Tku_MenuButton .menu "Query -under 0" \
    	"command -label {Query Class(s)} -command Query -accel Ctrl-Q -under 0" \
    	    {A query and update system} \
    	"com -label {Number of Instances} -com ShowCount -accel Ctrl-N -under 0" \
    	    {Get the number of instances in a class} \
    	"command -label {COPY to class} -command CopyIn -under 5" \
    	    {COPY tabular data from a file into the current class} \
    	"command -label {COPY from class} -command CopyOut -under 5" \
    	    {COPY tabular data from the current class to a file}
    
    Tku_MenuButton .menu "Options -under 0" \
    	"checkbutton -label {Include System Classes} -variable PGB(-system) -comm FillClasses -under 0" \
    	    {Include system classes in class list} \
    	"checkbutton -label {Verify Postquel} -variable PGB(-verify) -under 0" \
    	    {Modify and/or confirm Postquel before sending to the backend} \
    	"checkbutton -label {No Option Modifications} -variable PGB(-nomodify) -under 0" \
    	    {Do not allow modification of attriubte options and choices in the query tool}
    
    if $PGB(-debug) {
	foreach file [glob $PGB(Home)/lib/*.tcl] {
	    set file [file tail $file] 
	    lappend debugs \
    	    "command -label $file -command {DebugReSource $file}" ""
	}
	lappend debugs \
	    "command -label {Send Tk} -command Debug:SendTK" \
	    "Execute Tk code in pgbrowse's interpreter"
    	eval Tku_MenuButton .menu "{Debug -under 0}" $debugs
    }
    
#    frame .bottom
    pack .menu .info -fill x
    
    Tku_Listbox .dbs {GetClasses [selection get]} -geom 30x10
    Tku_Listbox .classes {Class open [selection get]} -geom 30x10
    pack forget .dbs.hs .classes.hs
    pack .dbs .classes -fill y -side left
    pack .classes -expand 1 -fill both -side left
    Tku_SetTabbing .dbs.listbox .classes.listbox
    tk_listboxSingleSelect .dbs.listbox
#    pack .bottom -expand 1 -fill both

    Tku_DefFocus .dbs.listbox
}

if $log LogOpen

# get the first dbs/classes and open the query window if needed
if $PGB(-hide) {
	set PGB(dbs) $db
	PQ sethost $host
	PQ setport $port
	PQ setdb $db
	set n 0
	foreach c [split $query ,] {
		lappend PGBC(classes) $c
		incr n
	}
	set PGBC(num) $n
	Query $c
} else {
	GetDbs $host $port
	if {$db != ""} {GetClasses $db}
	if {$class != ""} {
    		foreach c [split $class ,] {Class open $c}
	}
	if {$query != ""} {Query [split $query ,]}
}

unset db class query host port big nomodify debug system verify hide
