# 
# lobrowse.tcl - procedures for browsing large object filenames using pcd/pls
#
# $Header: /nfs/home/olive.s2k/src/base/local/tk/pgb3.0/lib/RCS/lobrowse.tcl,v 1.1 1993/12/06 20:04:16 jimbo Exp $
#

# ---------------------------------------------------------------------------
# LOStart - start the large object browser
# ---------------------------------------------------------------------------
proc LOStart {} {

    # check for an existing browser and a selected database
    if [winfo exists .lofd] {
    	Tku_Notice "Large object browser already active."
    	return 0
    }
    if ![CheckDB] return

    # export the current host/port/database to the environment
    PQ export

    # start the browser
    LOBrowse
}

# ---------------------------------------------------------------------------
# LOGetHier - return a directory hierarchy
# ---------------------------------------------------------------------------
proc LOGetHier {dir} { 
	if { $dir == "/" } { return "" }
	while {[set d [file dirname $dir]] != "/"} {lappend h $d}
	lappend h /
	return $h
}

# ---------------------------------------------------------------------------
# LOCwd	- change to a new directory
# ---------------------------------------------------------------------------
proc LOCwd { d } {
	set last [LOPwd]
	if ![LOCd $d] {Tku_Notice "Can't open directory $d."; return}
	LOUpdateDirs $last
}

# ---------------------------------------------------------------------------
# LOUpdateDirs - update current/last directory
# ---------------------------------------------------------------------------
proc LOUpdateDirs { last } {
	global LOvars
	set LOvars(lastdir) $last
	set LOvars(directory) [LOPwd]
	LOUpdate
}

# ---------------------------------------------------------------------------
# LOUpdate -	updates the large object listbox, file and glob entries
# ---------------------------------------------------------------------------
proc LOUpdate {} {
	global LOvars

	set dir [LOPwd]

	# if no glob pattern has been entered, focus on the entry box
	# so the user can add one
	if {$LOvars(glob) == "*" || $LOvars(glob) == ""} {
		set LOvars(all) 1
		.lofd.select.glob delete 0 end
	}

	# fill in the hierarchy menu button
	.lofd.box.menu configure -text $dir
	.lofd.box.menu.m delete 0 last
	foreach d [LOGetHier $dir] {
		.lofd.box.menu.m add command -label $d -command "LOCwd $d"
	}

	if [info exists LOvars(lastdir)] {
		if {$LOvars(lastdir) != "/" && $LOvars(lastdir) != $dir} {
			.lofd.box.menu.m add separator
			.lofd.box.menu.m add command -label $LOvars(lastdir) -command "LOCwd $LOvars(lastdir)"
		}
	}

	# clear and fill the list box
	Tku_ListboxClear .lofd.box.list
	update
##	if {$dir != "/"} {
##		set files ../
##	}
	if $LOvars(all) {
		if $LOvars(show) {set glob {.* *}} else {set glob *}
	} else {
		set glob $LOvars(glob)
	}

	# get list of files
	# NB: check globbing results because [glob zz] returns "zz" whether or
	# not file zz exists
##	if {$filelist == $glob && ![file exists $glob]} {set filelist ""}
##	foreach f [lsort $filelist] {
##		if {$f == "." || $f == ".."} continue
##		if [file isdirectory $f] {
##			lappend files $f/
##		} else {
##			if !$LOvars(SetDir) {lappend files $f}
##		}
##	}
	Tku_ListboxFill .lofd.box.list [LOGlob $glob]
}

# ---------------------------------------------------------------------------
# LOSelect -	narrows globbing to selected files
# ---------------------------------------------------------------------------
proc LOSelect {} {
	global LOvars
	set glob [.lofd.select.glob get]
	# focus on the glob entry box if nothings been entered
	if {$glob == ""} {
		focus .lofd.select.glob
	} else {
		set LOvars(all) 0
		set LOvars(glob) $glob
		LOUpdate
	}
}

# ---------------------------------------------------------------------------
# LOOk	- ok button action
# ---------------------------------------------------------------------------
proc LOOk { } {
	global LOvars

	# if nothings in the entry box, get the selected file
	if {[set file [.lofd.entry get]] ==""} {
		set i [.lofd.box.list curselection]
		case [llength $i] in {
			0	return
			1	{set file [.lofd.box.list get $i]}
			default	return
		}
	}
	set last [LOPwd]
	if [LOCd $file] {
		.lofd.entry delete 0 end
		LOUpdateDirs $last
		return
	}

	if $LOvars(SetDir) return

	if {[file dirname $file] != "."} {
		set dir [file dirname $file]
		LOCd $dir	
		set LOvars(directory) $dir
		set file [file tail $file]
	}

	case $LOvars(access) in {
	b* {
		# if browsing, just return file info
		LOInfo $file [LOLs -l "[lindex $file 0]"]
		return
	}
	w* {
		if [LOFile exists $file] {
##			if ![file writable $file] {Tku_Error "Can't write $file."; return}
			if ![Tku_Confirm "$file exists, overwrite?"] return
		}
##		if ![file writable $dir] {Tku_Error "Can't write in directory $dir"; return}
	   }
	r* {
		if ![LOFile exists $file] {Tku_Notice "$file does not exists."; return}
##		if ![file readable $file] {Tku_Error "Can't read $file."; return}
	    }
	}
	set LOvars(filename) $file
	destroy .lofd
}

# ---------------------------------------------------------------------------
# LOCancel -	cancel the large object browser
# ---------------------------------------------------------------------------
proc LOCancel {} {
	global LOvars
#	LOCd $LOvars(savecwd)
	set LOvars(filename) {}
	set LOvars(directory) {}
	destroy .lofd
}


# ---------------------------------------------------------------------------
# LO -		setup large object browser
# ---------------------------------------------------------------------------
# inputs:
#	various args:
#		-file <filename>	default filename
#		-dir <directory>	default directory
#		-select	<pattern>	glob files matching pattern
#		-title <title>		wm title
#		-access <mode>		access mode, read or write
#		-show			show hidden files
#		-setdir			get a directory instead of a filename
#		-open			return an open file descriptor
#					instead of just a valid filename
# ---------------------------------------------------------------------------
proc LO { args } {
	global LOvars env

	if {[set LOvars(LOCd) [Tku_SearchPath pcd]] == {}} {
		Tku_Error "can't find pcd in your path."
		return ""
	}
	if {[set LOvars(LOLs) [Tku_SearchPath pls]] == {}} {
		Tku_Error "can't find pls in your path."
		return ""
	}

	# set default arguements or restore arguments from last
	# file dialog
	set LOvars(savecwd) [LOPwd]
	if ![info exists LOvars(show)] {set LOvars(show) 0}
	if ![info exists LOvars(all)] {set LOvars(all) 1}
	if ![info exists LOvars(glob)] {set LOvars(glob) {}}
	set file {}
	set select {}
	set access r
	set title "Select a File"
	set dir {}

	# parse arguments
	if ![Tku_Args {:file :dir :select :access :title show open setdir} $args] return

	# update LOvars
	set LOvars(filename) $file
	if {$dir != {} && [LOFile isdirectory $dir]} {
		set LOvars(lastdir) $LOvars(savecwd); LOCd $dir
	} else {
		set dir $LOvars(savecwd)
	}
	
	if {$select != {}} {
		set LOvars(glob) $select
		set LOvars(all) 0
	}
	set LOvars(access) $access
	if $show {set LOvars(show) 1}
	set LOvars(directory) $dir
	set LOvars(SetDir) $setdir

	toplevel .lofd
	wm title .lofd $title
	wm minsize .lofd 1 1

	frame .lofd.bottom -bd 1 -relief raised
	# construct right-hand buttons
	frame .lofd.buttons
	if $LOvars(SetDir) {
		Tku_Button .lofd.ok {Set Directory} {destroy .lofd} default
		set label "Jump To..."
		set files directories
	} else {
		Tku_Button .lofd.ok OK LOOk default
		set label Filename
		set files files
	}
	Tku_Button .lofd.cancel Cancel LOCancel
	Tku_Button .lofd.refresh Update {LOUpdate}
	Tku_Button .lofd.home PGHOST... LOSetPGHost
	Tku_Button .lofd.db Database... LOSetPGDb

	pack .lofd.ok .lofd.cancel .lofd.refresh .lofd.home .lofd.db \
		-pady 5m -anchor w -fill x -in .lofd.buttons

	# construct left-hand labels, buttons, and listbox
	set width 40
	set LOvars(width) $width
	label .lofd.directory -anchor w -width $width
	LOTitle
	frame .lofd.box
	menubutton .lofd.menu -menu .lofd.menu.m -width $width -bd 3 \
		-relief raised -anchor w
	menu .lofd.menu.m
	listbox .lofd.list -yscroll {.lofd.scroll set} -bd 3 \
		-relief raised -geometry ${width}x10
	bind .lofd.list <Double-Button-1> {
		.lofd.entry delete 0 end
		.lofd.entry insert 0 [selection get]
		LOOk
	}
	scrollbar .lofd.scroll -command {.lofd.list yview} \
		-bd 2 -relief raised

	pack .lofd.menu -anchor w -fill x -in .lofd.box
	pack .lofd.list -side left -expand 1 -fill both -in .lofd.box
	pack .lofd.scroll -side left -fill y -in .lofd.box

	checkbutton .lofd.show -variable LOvars(show) -relief flat \
		-variable LOvars(show) -onvalue 1 -offvalue 0 \
		-text "Show hidden $files" -command LOUpdate

	label .lofd.filename -text $label -width $width -anchor w
	entry .lofd.entry -relief sunken -bd 4 -width $width
	.lofd.entry insert 0 $LOvars(filename)

	# construct file-pattern matching frame
	frame .lofd.select
	label .lofd.select.label -text "Select" -anchor e
	radiobutton .lofd.select.all -variable LOvars(all) -value 1 \
		-text "all $files" -command LOUpdate -relief flat
	radiobutton .lofd.select.match -variable LOvars(all) -value 0 \
		-text "$files matching" -command LOSelect -relief flat
	entry .lofd.select.glob -width 10 -relief sunken -bd 2
	.lofd.select.glob insert 0 $LOvars(glob)
	pack append .lofd.select \
		.lofd.select.label left \
		.lofd.select.all {top frame w} \
		.lofd.select.match left \
		.lofd.select.glob left
	bind .lofd.select.glob <Return> LOSelect

	set tablist ".lofd.entry .lofd.select.glob .lofd.ok.b .lofd.cancel.b .lofd.refresh.b .lofd.home.b"

	if !$LOvars(SetDir) {
		Tku_Button .lofd.jump "Jump To..." {LOCwd [Tku_Entry "Jump to directory" {} Jump]}
		pack append .lofd.b .lofd.jump { top pady 5m fillx } 
		bind .lofd.entry <Return> {.lofd.ok flash;.lofd.ok invoke}
		lappend tablist .lofd.jump.b
	} else {
		bind .lofd.entry <Return> LOOk
	}

	Tku_StatusBar .lofd.title $title

	pack .lofd.buttons -side right -padx 5m -in .lofd.bottom
	pack .lofd.directory -side top -anchor w -pady 1m -in .lofd.bottom
	pack .lofd.box -side top -anchor w -pady 1m -expand 1 -fill both \
		-in .lofd.bottom
	pack .lofd.show .lofd.filename .lofd.entry -side top -anchor w \
		-pady 1m -in .lofd.bottom
	pack .lofd.select -side top -anchor w -in .lofd.bottom

	pack .lofd.title .lofd.bottom -fill x -expand 1

	eval Tku_SetTabbing $tablist

	# update listbox and wait for .lofd to be destroyed
	LOUpdate
	bind Entry <Control-x> {
		puts stderr "\n\tLOvars:\n[join [Tku_DumpArray LOvars] \n]"
	}
	tkwait window .lofd
	# cancel'ed early return
	if {$LOvars(directory) == {}} {return {}}

##	LOCd $LOvars(savecwd)

	# directory early return
	if $LOvars(SetDir) {
		return $LOvars(directory)
	}

	# filename-only early return
	if !$open {return $LOvars(directory)/$LOvars(filename)}

##	# attempt to open file and return file descriptor
##	if [catch {set file [open $LOvars(directory)/$LOvars(filename) $LOvars(access)]} err] {
##		Tku_Error $err ; return {}
##	} else {
##		return $file
##	}
}

# ---------------------------------------------------------------------------
# LO macros -	simple macros for calling LO
#
# title	wm title
# args	additional LO args (e.g., -file, -show, etc.)
# ---------------------------------------------------------------------------
# Return a writable filename
# ---------------------------------------------------------------------------
proc LOSaveFilename {args}  {
	eval LO -access w -title {{Save File}} $args
}
# ---------------------------------------------------------------------------
# Return a readable filename
# ---------------------------------------------------------------------------
proc LOOpenFilename {args} {
	eval LO -access r -title {{Open File}} $args
}
proc LOBrowse {args} {
	eval LO -access b -title {{Large Object Browser}} $args
}
# ---------------------------------------------------------------------------
# Return a directory name
# ---------------------------------------------------------------------------
proc LOSetDir {args} {
	eval LO -setdir -title {{Set Directory}} $args
}

# ---------------------------------------------------------------------------
# LOCd - attempt to change to a new large object directory
# ---------------------------------------------------------------------------
proc LOCd {dir} {
	global env LOvars
	Tku_Working Scanning...
	LOPwd
	if [catch {set result [exec $LOvars(LOCd) -D $env(DATABASE) $dir]}] {
		return 0
	}
	set env(PFCWD) [lindex $result 2]
	Tku_Waiting .lofd.title
	return 1
}		

# ---------------------------------------------------------------------------
# LOPwd - return current large object directory
# ---------------------------------------------------------------------------
proc LOPwd {} {
	global env
	if ![info exists env(PFCWD)] {
		set env(PFCWD) /
		return /
	} else {
		return $env(PFCWD)
	}
}

# ---------------------------------------------------------------------------
# LOFile - test aspects of large objects
# ---------------------------------------------------------------------------
proc LOFile {cmd file} {
	global LOvars
	Tku_Working Scanning...
	case $cmd in {
		exists {
			if {[LOGlob $file] == {}} {
				return 0
			} else {
				return 1
			}
		}
		isdirectory {
			set save [LOPwd]
			if [LOCd $file] {
				LOCd $save
				return 1
			} else {
				return 0
			}
		}
	}
	Tku_Waiting
}

# ---------------------------------------------------------------------------
# LOGlob - simulate file globbing on large objects
# ---------------------------------------------------------------------------
proc LOGlob {pattern} {
	global LOvars
	Tku_Working Scanning...
	if [regexp {^/} $pattern] {
		set sPcwd [LOPwd]
		LOCd [file dirname $pattern]
		set files [LOGlob [file tail $pattern]]
		LOCd $sPcwd
		return $files
	} else {
		set files [split [LOLs] \n]
		foreach file $files {
			if [string match $pattern $file] {
				lappend results $file
			}
		}
		if ![info exists results] {set results {}}
		return $results
	}
	Tku_Waiting
}

# ---------------------------------------------------------------------------
# LOLs - simulate the file ls command for large objects
# ---------------------------------------------------------------------------
proc LOLs {args} {
	global LOvars env
	Tku_Working Scanning...
	if [catch {set ret [eval exec $LOvars(LOLs) -1 $args]} err] {
		Tku_Error "Could not get LO file list:\n$err"
		set LOvars(status) 0
		Tku_Waiting
		return ""
	} else {
		Tku_Waiting
		set LOvars(status) 1
		return $ret
	}
}

# ---------------------------------------------------------------------------
# LOSetPGHost - set postgres host for the large object browser
# ---------------------------------------------------------------------------
proc LOSetPGHost {} {
	global env
	if {[set host [Tku_Entry "PGHOST"]] == {}} return
	set env(PGHOST) $host
	LOTitle
	LOCd /
	LOUpdate
}

# ---------------------------------------------------------------------------
# LOSetPGHost - set postgres database for the large object browser
# ---------------------------------------------------------------------------
proc LOSetPGDb {} {
	global env
	if {[set db [Tku_Entry "Database"]] == {}} return
	set env(DATABASE) $db
	LOTitle
	LOCd /
	LOUpdate
}

# ---------------------------------------------------------------------------
# LOTitle - update title of large object browser
# ---------------------------------------------------------------------------
proc LOTitle {} {
	global env
	.lofd.directory conf \
		-text "Database:  $env(DATABASE)  Host:  $env(PGHOST)"
}

# ---------------------------------------------------------------------------
# LOInfo - popup a box with simple information on a large object
# ---------------------------------------------------------------------------
proc LOInfo {file ll} {
	global env LOvars
	if !$LOvars(status) return
	toplevel .i
	wm title .i "Large Object Info"
	set bytes [lindex $ll 4]
	if {$bytes > 10000} {
		if {$bytes > 10000000} {
			set bytes [expr $bytes/1002400]M
		} else {
			set bytes [expr $bytes/1024]K
		}
	}
	message .i.m -text \
"Host:\t\t$env(PGHOST)\nDatabase:\t$env(DATABASE)\nDirectory:\t[LOPwd]\nFile:\t\t$file\nSize:\t\t$bytes bytes\n\n$ll" \
		-bd 2 -relief sunken -width 5i
	Tku_Button .i.b OK {destroy .i}
	pack append .i \
		.i.m {top} \
		.i.b {top pady 5m}
}


