#
# support.tcl - procedures for adding/modifying the pgb_support class
#
# $Header: /nfs/home/olive.s2k/src/base/local/tk/pgb3.0/lib/RCS/support.tcl,v 1.1 1993/12/06 20:04:39 jimbo Exp $
#

# ---------------------------------------------------------------------------
# VerifyPGBSupport - look for and create the pgb_support class,
#	index, and access function if it doesn't exist in the
#	database
# ---------------------------------------------------------------------------
proc VerifyPGBSupport {} {

    # look for the pgb_support class and pgb_namex index in pg_class
    set cmd {
retrieve (
    pg_class.relname
)
    where pg_class.relname = "pgb_support"
    or    pg_class.relname = "pgb_namex"
}
    if ![PQ exec $cmd "Checking support..."] {
	PQ errbox "Couldn't check for the pgb_support class and index."
	return 0
    }
    if {[lsearch [PQ getcol 0] pgb_support] != -1} \
	{set pgb_support 1} else {set pgb_support 0}
    if {[lsearch [PQ getcol 0] pgb_namex] != -1} \
	{set pgb_namex 1} else {set pgb_namex 0}

    # look for the pgb_get_meta function
    set cmd {retrieve (pg_proc.proname) where pg_proc.proname = "pgb_get_meta"}
    if ![PQ exec $cmd "Checking support..."] {
	PQ errbox "Could not check for the pgb_get_meta function."
	return 0
    }
    if {[lsearch [PQ getcol 0] pgb_get_meta] != -1} \
	{set pgb_get_meta 1} else {set pgb_get_meta 0}

    # create pgb_support if needed
    if !$pgb_support {
	set cmd {
create pgb_support (
    pgb_class = char16,
    pgb_attribute = char16,
    pgb_metadata = text
)
}
        if ![PQ exec $cmd "Creating support class..."] {
	    PQ errbox "Could not create pgb_support class."
	    return 0
        }
	# delete any existing pgb_namex index
	if $pgb_namex {
	    if ![PQ exec "remove index pgb_namex" "Updating support..."] {
		PQ errbox "Could not delete existing pgb_namex index."
		return 0
	    }
	    set pgb_namex 0
	}
	# delete any existing pgb_get_meta function
	if $pgb_get_meta {
	    if ![PQ exec "remove function pgb_get_meta" "Updating support..."] {
		PQ errbox "Could not delete existing pgb_get_meta function."
		return 0
	    }
	    set pgb_get_meta 0
	}
    }

    # create or re-create pgb_namex index
    if !$pgb_namex {
	set cmd {
define index pgb_namex
    on pgb_support
    using btree (pgb_class char16_ops)
}
        if ![PQ exec $cmd "Updating support..."] {
	    PQ errbox "Could not create the pgb_namex index."
	    return 0
	}
    }

    # create or re-create the pgb_get_meta function
    if !$pgb_get_meta {
	set cmd {
define function pgb_get_meta
   (language="postquel", returntype=text) arg is (char16, char16)
   as "retrieve (pgb_support.pgb_metadata) where pgb_support.pgb_class = $1 and   pgb_support.pgb_attribute = $2" 
}
	if ![PQ exec $cmd "Updating support..."] {
	    PQ errbox "Could not add pgb_get_meta function."
	    return 0
        }
    }
    return 1
}

# ---------------------------------------------------------------------------
# PGBSupportUpdate - replace or append metadata for an attribute
# ---------------------------------------------------------------------------
proc PGBSupportUpdate {class att metadata} {
    if {[Attribute meta $class $att] == "(null)"} {
	PGBSupportAppend $class $att $metadata
    } else {
	PGBSupportReplace $class $att $metadata
    }
}

# ---------------------------------------------------------------------------
# PGBSupportReplace - replace a pgb_support instance
# ---------------------------------------------------------------------------
proc PGBSupportReplace {class att metadata} {
    set cmd "
replace pgb_support (
    pgb_metadata = \"$metadata\"
)   where pgb_support.pgb_class = \"$class\"
    and   pgb_support.pgb_attribute = \"$att\"
"
    if ![PQ exec $cmd "Updating support..."] {
	PQ errbox "Could not update metadata."
	return 0
    }
    return 1
}

# ---------------------------------------------------------------------------
# PGBSupportReplace - add a pgb_support instance
# ---------------------------------------------------------------------------
proc PGBSupportAppend {class att metadata} {

    PGBSupportDelete $class $att
    
    set cmd "
append pgb_support (
    pgb_class = \"$class\",
    pgb_attribute = \"$att\",
    pgb_metadata = \"$metadata\"
)"
    if ![PQ exec $cmd "Updating support..."] {
	PQ errbox "Could not append metadata."
	return 0
    }
    return 1
}

# ---------------------------------------------------------------------------
# PGBSupportReplace - add a pgb_support instance
# ---------------------------------------------------------------------------
proc PGBSupportDelete {class {att ""}} {
    set cmd "
delete pgb_support
where pgb_support.pgb_class = \"$class\"
"
    if {$att != ""} {
	append cmd "and   pgb_support.pgb_attribute = \"$att\""
    }

    if ![PQ exec $cmd "Clearing support..."] {
	PQ errbox "Could not clear existing metadata."
	return 0
    }
}
